#ifndef AUDIO_SOUND_H
#define AUDIO_SOUND_H TRUE

/*
**    sound.h
**
**    (C) Copyright 2002-2003 Rocklyte Systems.
**        All Rights Reserved.
**
*/

#ifndef MAIN_H
#include <pandora/main.h>
#endif

/*****************************************************************************
** Sound class.
*/

#define VER_SOUND ((FLOAT)1.0)

struct Sound {
   OBJECT_HEADER             /* Standard structure header */
   LONG      Priority;       /* Priority over other sounds */
   LONG      Length;         /* Length of sample data in bytes (also refer BufferLength) */
   LONG      Octave;         /* Current octave to use for playing back notes (defaults to zero) */
   FLOAT     Volume;         /* Volume of sample (0 - 100%) */
   LONG      Flags;          /* Sound flags */
   LONG      Frequency;      /* Frequency of sampled sound (nb: does not affect playback - use the Playback field) */
   LONG      Playback;       /* Frequency to use for sample playback */
   FLOAT     Pan;            /* Horizontal positioning for playback on stereo speakers (-100 to +100) */
   LONG      Compression;    /* Compression rating (0% none, 100% high) */
   LONG      BytesPerSecond; /* Bytes per second (Formula: Frequency * BytesPerSample) */
   LONG      BitsPerSample;  /* Usually set to 8 or 16 bit */
   LONG      Static;         /* Set to TRUE if the object should be static */
   OBJECTID  AudioID;        /* Reference to an Audio object to use for audio output and input */
   LONG      LoopStart;      /* Byte position of looping start */
   LONG      LoopEnd;        /* Byte position of looping end */
   LONG      Stream;         /* Streaming type (smart by default) */
   LONG      BufferLength;   /* Length of audio buffer in bytes (relevant if streaming) */
   OBJECTID  StreamID;       /* Object allocated for streaming */
   LONG      Position;       /* Byte position to start playing from */
   LONG      Handle;         /* Audio handle acquired at the audio object [Private - Available to child classes] */
   LONG      ChannelIndex;   /* [Private - Available to child classes] */
   OBJECTPTR File;           /* The file holding the sample data [Private - Available to child classes]  */

   PRIVATE_FIELDS
   STRING   Fields[30];
   UBYTE    prvHeader[128];
   LONG     prvFormat;         /* The format of the sound data */
   LONG     prvDataOffset;     /* Start of raw audio data within the source file */
   STRING   prvLocation;
   STRING   prvDescription;
   STRING   prvDisclaimer;
   LONG     prvNote;               /* Note to play back (e.g. C, C#, G...) */
   UBYTE    prvNoteString[4];
   struct WAVEFormat *prvWAVE;
   UBYTE    prvPlatformData[128];  /* Data area for holding platform/hardware specific information */
   LONG     prvAlignment;      /* Byte alignment value */
};

typedef struct WAVEFormat {
   WORD Format;             /* Type of WAVE data in the chunk */
   WORD Channels;           /* Number of channels, 1=mono, 2=stereo */
   LONG Frequency;          /* Playback frequency */
   LONG AvgBytesPerSecond;  /* Channels * SamplesPerSecond * (BitsPerSample / 8) */
   WORD BlockAlign;         /* Channels * (BitsPerSample / 8) */
   WORD BitsPerSample;      /* Bits per sample */
   WORD ExtraLength;
} WAVEFORMATEX;

/*****************************************************************************
** Sound Flags
*/

#define SDF_LOOP         0x00000001    /* Repeat sample forever / allow programmed sample looping */
#define SDF_NEW          0x00000002    /* Create the sample from scratch */
#define SDF_QUERY        0x00000004    /* Use this flag when querying sound files rather than loading them */
#define SDF_STEREO       0x00000008    /* Indicates that the sound has multiple audio channels */
#define SDF_TERMINATE    0x00000020    /* Terminate when sound completes playing */
#define SDF_RESTRICTPLAY 0x00000040    /* Restricts playback so that the sound can never play on more than one channel at any given time */

#define SDF_STREAM       0x40000000    /* Read-Only: Indicates that streaming is enabled */
#define SDF_NOTE         0x80000000    /* Used internally for managing playback */

/*****************************************************************************
** Streaming options.
*/

#define STREAM_NEVER  1  /* No streaming - load all data into memory */
#define STREAM_SMART  2  /* Stream only if sample exceeds 512k */
#define STREAM_ALWAYS 3  /* Stream if sample exceeds 64k */

/*****************************************************************************
** Definitions for the Note field.  An 'S' indicates a sharp note.
*/

#define NOTE_C   0
#define NOTE_CS  1
#define NOTE_D   2
#define NOTE_DS  3
#define NOTE_E   4
#define NOTE_F   5
#define NOTE_FS  6
#define NOTE_G   7
#define NOTE_GS  8
#define NOTE_A   9
#define NOTE_AS 10
#define NOTE_B  11

#define NOTE_OCTAVE 12

#endif /* SOUND_SOUND_H */
