#ifndef GRAPHICS_POINTER_H
#define GRAPHICS_POINTER_H

/*
**  pointer.h
**
**  (C) Copyright 2002-2004 Rocklyte Systems.
**      All Rights Reserved.
*/

#ifndef MAIN_H
#include <pandora/main.h>
#endif

/*****************************************************************************
** Predefined cursor styles.
*/

enum {
   PTR_NOCHANGE = 0,
   PTR_DEFAULT,
   PTR_SIZE_BOTTOM_LEFT,
   PTR_SIZE_BOTTOM_RIGHT,
   PTR_SIZE_TOP_LEFT,
   PTR_SIZE_TOP_RIGHT,
   PTR_SIZE_LEFT,
   PTR_SIZE_RIGHT,
   PTR_SIZE_TOP,
   PTR_SIZE_BOTTOM,
   PTR_CROSSHAIR,
   PTR_SLEEP,
   PTR_SIZING,
   PTR_SPLIT_VERTICAL,
   PTR_SPLIT_HORIZONTAL,
   PTR_MAGNIFIER,
   PTR_HAND,
   PTR_HAND_LEFT,
   PTR_HAND_RIGHT,
   PTR_TEXT,
   PTR_PAINTBRUSH,
   PTR_STOP,
   PTR_INVISIBLE,
   PTR_CUSTOM,
   /* Any new cursor images must be added to the CursorLookup array in pointer_class.c */
   PTR_END
};

/*****************************************************************************
** Pointer class
*/

struct Pointer {
   OBJECT_HEADER             /* Standard header structure */
   LONG     XCoord;          /* Current x coordinate */
   LONG     YCoord;          /* Current y coordinate */
   FLOAT    Speed;           /* Speed multiplier (%) */
   FLOAT    Acceleration;    /* Acceleration level - keep between 0.0 and 3.0 */
   LONG     MaxSpeed;        /* Maximum speed */
   LONG     OverX;           /* The X coord relative to the closest object */
   LONG     OverY;           /* The Y coord relative to the closest object */
   LONG     OverZ;           /* The Z distance relative to the closest object */
   OBJECTID InputID;         /* Indicates where pointer input comes from (usually SystemMouse) */
   OBJECTID DrawableID;      /* Top-most drawable that contains the pointer */
   OBJECTID AnchorID;        /* Drawable object that the pointer is anchored to */
   LONG     CursorID;
   OBJECTID CursorOwnerID;
   FLOAT    DoubleClick;
   FLOAT    WheelSpeed;
   LONG     Flags;
   OBJECTID RestrictID;      /* Drawable object that the pointer is restricted to */
   LONG     HostX;           /* Indicates the current position of the host cursor on Windows or X11 */
   LONG     HostY;           /* Indicates the current position of the host cursor on Windows or X11 */
   OBJECTID BitmapID;        /* Bitmap to use for custom cursor images */

   PRIVATE_FIELDS
   OBJECTID LastRenderID;    /* Last object that the pointer was positioned over */
   OBJECTID LastClickedB1;   /* LMB - last click */
   OBJECTID LastClickedB2;   /* RMB - last click */
   OBJECTID LastClickedB3;   /* MMB - last click */
   OBJECTID OverObjectID;
   OBJECTID CursorDrawableID;
   OBJECTID KeyboardID;
   OBJECTID CursorReleaseID;
   OBJECTID BitmapThemeID;
   MEMORYID MessageQueue;       /* Message port of the task that holds the cursor */
   LONG     LastClickX;
   LONG     LastClickY;
   LONG     LastReleaseX;
   LONG     LastReleaseY;
   LONG     RepeatDelayB1;
   LONG     RepeatDelayB2;
   LONG     RepeatDelayB3;
   LONG     CursorRelease;
   LARGE    ClickTime;
   struct   Time *prvTime;
   LONG     BufferCursor;
   LONG     BufferFlags;
   MEMORYID BufferQueue;
   OBJECTID BufferOwner;
   OBJECTID BufferObject;
   UBYTE    Device[32];
   UBYTE    Theme[32];
   struct {
      WORD HotX;
      WORD HotY;
   } Cursors[PTR_END];
};

#define IOF_ADD      0x00000001 /* Add a + symbol to the bottom right of the image */
#define IOF_SUBTRACT 0x00000002 /* Add a - symbol to the bottom right of the image */
#define IOF_QUESTION 0x00000004 /* Add a ? symbol to the bottom right of the image */

#define PF_SOFTWARE 0x00000001 /* The pointer is software, not hardware based */
#define PF_VISIBLE  0x00000002 /* The pointer is currently visible */
#define PF_ANCHOR   0x00000004 /* Allow the pointer to be anchored */

/*****************************************************************************
** Pointer Methods.
*/

#define MT_SetCursor        -1
#define MT_SetPointerImage  -2
#define MT_RestoreCursor    -4
#define MT_SoftwareCursor   -5
#define MT_HardwareCursor   -6
#define MT_SetWinCursor     -7 /* Private, win32 only */
#define MT_UngrabX11Pointer -8 /* Private, X11 only */
#define MT_GrabX11Pointer   -9 /* Private, X11 only */

#define MT_ReleaseCursor    MT_RestoreCursor

struct mtSetCursor {
   OBJECTID ObjectID;
   LONG     Flags;
   LONG     Cursor;
   STRING   Name;
   OBJECTID OwnerID;
   LONG     PreviousCursor;
};

#define ACF_LMB       0x00000001
#define ACF_MMB       0x00000002
#define ACF_RMB       0x00000004
#define ACF_ANCHOR    0x00000008
#define ACF_RESTRICT  0x00000010
#define ACF_BUFFER    0x00000020
#define ACF_MOVE      0x00000040
#define ACF_NOBUTTONS 0x00000080

struct mtRestoreCursor {
   LONG     Cursor;
   OBJECTID OwnerID;
};

struct mtSetPointerImage {
   OBJECTID BitmapID;       /* Image object to set (pointer will be extracted from the area defined by ClipX, ClipY etc) */
   OBJECTID OwnerID;
   LONG HotX;
   LONG HotY;
};

struct mtGrabX11Pointer {
   OBJECTID DrawableID;
};

struct mtSetWinCursor {
   LONG Cursor;
};

#endif /* GRAPHICS_POINTER_H */
