#ifndef MAIN_H
#define MAIN_H TRUE

/*
**   main.h
**
**   General include file for all programs.
**
**   (C) Copyright 1996-2004 Rocklyte Systems.
**       All Rights Reserved.
*/

#ifndef SYSTEM_TYPES_H
#include <pandora/system/types.h>
#endif

#ifndef SYSTEM_REGISTRY_H
#include <pandora/system/registry.h>
#endif

#ifndef SYSTEM_ERRORS_H
#include <pandora/system/errors.h>
#endif

/***************************************************************************/

#define VER_KERNEL ((FLOAT)3.4)  /* Kernel version + revision */
#define REV_KERNEL (4)           /* Kernel revision as a whole number*/

#ifdef _WIN32
#define __export __declspec(dllexport)
#else
#define __export
#endif

#define EXPORT __export

//#define __pentium__   /* Is this a Pentium CPU? (auto-defined by GCC, use -mpentium) */
#define __x86__         /* Does the CPU support the x86 instruction set? (i386 bottom-line) */
#define __kernelbase__  /* Use KernelBase to make function calls */
#define __mission__     /* Mission critical mode */
#define CPU_PC CPU_I586 /* What is the minimum required CPU for the compiled code? */
#define REVERSE_BYTEORDER TRUE      /* Reverse byte order / little endian (true for Intel CPU's) */

#define PRIVATE_FIELDS

#undef  NULL    /* Turn off any previous definition of NULL */
#define NULL 0  /* NULL is a value of 0 */

#define skipwhitespace(a) while ((*(a) > 0) && (*(a) <= 0x20)) (a)++;

#define ARRAYSIZE(a) (sizeof(a)/sizeof(a[0]))

#undef MIN
#undef MAX
#undef MID
#undef ABS

#define ROUNDUP(a,b) (((a) + (b)) - ((a) % (b))) /* ROUNDUP(Number, Alignment) e.g. (14,8) = 16 */
#define MIN(x,y)     (((x) < (y)) ? (x) : (y))
#define MAX(x,y)     (((x) > (y)) ? (x) : (y))
#define MID(x,y,z)   MAX((x), MIN((y), (z)))
#define ABS(x)       (((x) >= 0) ? (x) : (-(x)))

#define lcase(a) ((((a) >= 'A') AND ((a) <= 'Z')) ? ((a) - 'A' + 'a') : (a))

#define AlignLong(a) ((a) + 3) & (~3)
#define AlignWord(a) ((a) + 1) & (~1)

/*****************************************************************************
** Endian management routines.
*/

#ifdef REVERSE_BYTEORDER
#define rd_long(a)   (((a)[0]<<24)|((a)[1]<<16)|((a)[2]<<8)|((a)[3]))
#define rd_word(a)   (((a)[0]<<8)|((a)[1]))
#define wrb_long(a,b) ((LONG *)(b))[0] = (a)
#define wrb_word(a,b) ((WORD *)(b))[0] = (a)
#else
#define rd_long(a)   (a)
#define rd_word(a)   (a)
#define wd_long(a,b) ((LONG *)(b))[0] = (a)
#define wd_word(a,b) ((WORD *)(b))[0] = (a)
#define wrb_long(a,b) (b)[0] = (UBYTE)(a); (b)[1] = (UBYTE)((a)>>8); (b)[2] = (UBYTE)((a)>>16); (b)[3] = (UBYTE)((a)>>24)
#define wrb_word(a,b) (b)[0] = (UBYTE)(a); (b)[1] = (UBYTE)((a)>>8)
#endif

/*****************************************************************************
** Structure to pass to OpenKernel()
*/

struct OpenInfo {
   LONG   Flags;
   STRING Name;
   STRING Copyright;
   STRING Date;
   STRING Author;
   FLOAT  KernelVersion;
   LONG   JumpTable;
   LONG   MaxStep;
   LONG   Detail;
   STRING *Args;
   LONG   ArgCount;
   ERROR  Error;
   FLOAT  CompiledAgainst;
};

#define OPF_NAME            0x00000001
#define OPF_COPYRIGHT       0x00000002
#define OPF_DATE            0x00000004
#define OPF_AUTHOR          0x00000008
#define OPF_KERNELVERSION   0x00000010
#define OPF_JUMPTABLE       0x00000020
#define OPF_MAXSTEP         0x00000040
#define OPF_DETAIL          0x00000080
#define OPF_SHOWMEMORY      0x00000100
#define OPF_SHOWIO          0x00000200
#define OPF_SHOWERRORS      0x00000400
#define OPF_ARGS            0x00000800
#define OPF_ERROR           0x00001000
#define OPF_COMPILEDAGAINST 0x00002000

/****************************************************************************
** Tag definitions.
*/

#define TAGEND    0
#define TPTR      (1L<<30)
#define TFLOAT    (1L<<29)
#define TDOUBLE   (1L<<28)
#define TSTR      (1L<<27)
#define TLONG     (1L<<26)
#define TLARGE    (1L<<25)
#define TUNLISTED (1L<<24)
#define TTRANS    (1L<<23)

#define TSTRING  (TSTR)
#define TALLTAGS (TLONG|TPTR|TFLOAT|TDOUBLE|TLARGE|TSTR|TUNLISTED|TTRANS)

/*****************************************************************************
** Header used for all objects.
*/

struct Head {
   CLASSID   ClassID;      /* Reference to the object's class, used to resolve the Class pointer */
   CLASSID   SubID;        /* Reference to the object's sub-class, used to resolve the Class pointer */
   OBJECTID  UniqueID;     /* Unique object identifier */
   OBJECTID  OwnerID;      /* Refers to the owner of this object */
   WORD      ObjectFlags;  /* Object flags */
   WORD      MemFlags;     /* Recommended memory allocation flags */
   OBJECTPTR Class;        /* Class pointer, resolved on AccessObject() */
   struct Stats *Stats;    /* Stats pointer, resolved on AccessObject() [Private] */
   OBJECTID TaskID;        /* The Task that this object belongs to */
};

#define ClassName(a) ((struct SysClass *)(((OBJECTPTR)(a))->Class))->Name

#define OBJECT_HEADER struct Head Head;

/*****************************************************************************
** These structures are private.  Never, ever access them.
*/

#define MAX_NAMELEN 28

struct Stats {
   MEMORYID MID_ChannelList;         /* Array of objects that are listening for data (struct ChannelSubscription) */
   MEMORYID MID_ChildPrivate;        /* A reserved field for use by child classes */
   MEMORYID MID_ActionSubscriptions; /* Action subscriptions (struct ActionSubscription) */
   MEMORYID MID_FieldSubscriptions;  /* Field monitoring array (struct FieldSubscription) */
   UBYTE    Name[MAX_NAMELEN];       /* The name of the object (optional) */
   LONG     NotifyFlags[4];          /* Notification flags - space for 128 actions max */
   LONG     FieldFlags[4];           /* Field notification flags - space for 128 fields max */
   APTR     ChildPrivate;            /* Address for the ChildPrivate structure, if allocated */
   WORD     SubscriptionSize;        /* Size of the subscription array */
   WORD     ChannelSize;             /* Size of the channel array */
   WORD     FieldSize;               /* Size of the field subscription array */
   WORD     empty;                   /* alignment */
   LONG     MethodFlags[2];          /* Method flags - space for 64 methods max */
};

/*** Subscription structures ***/

struct ChannelSubscription {
   OBJECTID SubscriberID;    /* Subscriber ID */
   MEMORYID MessagePortMID;  /* Message port for the subscriber */
   CLASSID  ClassID;         /* Class of the subscriber */
};

struct ActionSubscription {
   ACTIONID ActionID;       /* Monitored action */
   OBJECTID SubscriberID;   /* Object to be notified */
   MEMORYID MessagePortMID; /* Message port for the object */
   CLASSID  ClassID;        /* Class of the subscribed object */
};

struct FieldSubscription {
   FIELD    FieldID;        /* Monitored field */
   OBJECTID SubscriberID;   /* Object to be notified */
   MEMORYID MessagePortMID; /* Message port for the object */
   CLASSID  ClassID;        /* Class of the subscribed object */
};

/****************************************************************************
** Messaging structures.
*/

struct Message { /* Note: This structure is utilised by struct NextMsg */
   LONG UniqueID;
   LONG Type;
   LONG Size;
   /*** Data follows ***/
};

#define MF_WAIT          0x00000001
#define MF_CLEAR         0x00000002
#define MF_QUERYUSER     0x00000004
#define MF_QUERYSYSTEM   0x00000008
#define MF_UPDATE        0x00000010
#define MF_NODUPLICATE   0x00000020
#define MF_ADD           0x00000040
#define MF_ADDRESS       0x00000080
#define MF_NOMSGHOOK     0x00000100
#define MF_TIMEOUT       0x00000200
#define MF_IGNOREZOMBIES 0x00000400
#define MF_MESSAGEID     0x00000800

#define MF_QUERY (MF_QUERYUSER|MF_QUERYSYSTEM)

/*** Reserved message ID's handled internally by the Pandora Engine ***/

#define MSGID_ACTION       1
#define MSGID_SETFIELD     2
#define MSGID_GETFIELD     3
#define MSGID_ACTIONRESULT 5
#define MSGID_DEBUG        6
#define MSGID_EXPOSE       7
#define MSGID_SYSTEMEND    1000 /* End marker for system messages */

/*** Reserved message ID's that require a customised user response ***/

#define MSGID_QUIT         1000
#define MSGID_SERVICE      1001

struct ExposeMessage {
   OBJECTID ObjectID;
   LONG XCoord;
   LONG YCoord;
   LONG Width;
   LONG Height;
   LONG Flags;
};

struct DebugMessage {
   LONG DebugID;
};

/****************************************************************************
** Flags that can be passed to NewObject().
*/

#define NF_NOTRACK       0x0001  /* Resource tracking can be turned off for an object if required */
#define NF_PUBLIC        0x0002  /* Public objects can be accessed by any task */
#define NF_NETWORK       0x0004  /* Gives permissions for object sharing over a network/internet */
#define NF_INITIALISED   0x0008  /* Set if the object has been initialised */
#define NF_CHILD         0x0010  /* Set if the object should not appear in queries */
#define NF_EXCLUSIVEFREE 0x0020  /* Set if the object is marked for deletion */
#define NF_FREE          0x0040  /* Indicates that the object is being freed */
#define NF_NOTIFYMARKER  0x0080  /* Private */
#define NF_HEALTHCHECK   0x0100  /* Set if the object requires a health-check */
#define NF_SHARED        0x0200  /* Set if other tasks are allowed to find the object */
#define NF_LOCKED        0x0400  /* Set if the object is locked by AccessObject() */
#define NF_FREEMARK      0x0800  /* Private */
#define NF_NEWOBJECT     0x1000  /* Private: The object is in the process of being created */

#define NF_UNIQUE        0x40000000  /* Do not create the object if one already exists with an identical name */
#define NF_NAME          0x80000000

/*** Synonyms ***/

#define NF_NETWORKED  (NF_NETWORK)
#define NF_NOTRACKING (NF_NOTRACK)
#define NF_UNTRACKED  (NF_NOTRACK)
#define NF_PRIVATE    (NULL)

/*****************************************************************************
** Argument flags for defining methods, actions and functions.
*/

#define ARG_LONG    0x0001
#define ARG_LARGE   0x0002
#define ARG_OBJECT  0x0004  /* Supplementary */
#define ARG_RESULT  0x0800  /* Supplementary */
#define ARG_DOUBLE  0x1000
#define ARG_PTRSIZE 0x2000  /* Supplementary */
#define ARG_STR     0x4000
#define ARG_PTR     ((WORD)0x8000)

#define ARG_VOID         (ARG_LONG)
#define ARG_ERROR        (ARG_LONG)
#define ARG_STRING       (ARG_STR)
#define ARG_LONGRESULT   (ARG_LONG|ARG_PTR|ARG_RESULT)   /* NOTE: Use for pointer-based value results only */
#define ARG_LARGERESULT  (ARG_LARGE|ARG_PTR|ARG_RESULT)  /* NOTE: Use for pointer-based value results only */
#define ARG_DOUBLERESULT (ARG_DOUBLE|ARG_PTR|ARG_RESULT) /* NOTE: Use for pointer-based value results only */
#define ARG_PTRRESULT    (ARG_PTR|ARG_RESULT)            /* NOTE: Use for pointer-based value results only */

/*****************************************************************************
** Memory types used by AllocMemory().  The lower 16 bits are stored with
** allocated blocks, the upper 16 bits are function-relative only.
*/

#define MEM_DATA      0x0000  /* Default */
#define MEM_PUBLIC    0x0001
#define MEM_VIDEO     0x0002
#define MEM_BLIT      0x0004
#define MEM_AUDIO     0x0008
#define MEM_CODE      0x0010
#define MEM_NOPOOL    0x0020 /* Gives a hint to the allocator to allocate the block outside of the memory pool */
#define MEM_0040      0x0040
#define MEM_UNTRACKED 0x0080
#define MEM_STRING    0x0100
#define MEM_OBJECT    0x0200  /* Internal - the memory block represents an object structure */
#define MEM_0400      0x0400
#define MEM_0800      0x0800
#define MEM_EXCLUSIVE 0x1000  /* Internal */
#define MEM_DELETE    0x2000  /* Internal - the memory block is marked for deletion */
#define MEM_NOBLOCK   0x4000  /* Allow multiple accesses */
#define MEM_FIXED     0x8000  /* Allocate to a global fixed address */

#define MEM_READ        0x00010000
#define MEM_WRITE       0x00020000
#define MEM_NOCLEAR     0x00040000
#define MEM_RESERVED    0x00080000
#define MEM_HIDDEN      0x00100000
#define MEM_TASK        0x00200000
#define MEM_GLOBAL      0x00400000
#define MEM_CALLER      0x00800000 /* Used in method/action support routines to allocate memory against the caller */
#define MEM_EXECUTE     0x01000000
#define MEM_NOSIZELIMIT 0x02000000

#define MEM_SHARED     MEM_PUBLIC
#define MEM_READWRITE  (MEM_READ|MEM_WRITE)
#define MEM_NOBLOCKING MEM_NOBLOCK
#define MEM_EXECUTABLE MEM_EXECUTE

#define ResolveAddress(a,b)  ((APTR)(((BYTE *)(a)) + (b)))

/*****************************************************************************
** Universal dimension management flags
*/

#define DMF_RELATIVE_X       0x0001
#define DMF_RELATIVE_Y       0x0002
#define DMF_FIXED_X          0x0004
#define DMF_FIXED_Y          0x0008
#define DMF_RELATIVE_XOFFSET 0x0010
#define DMF_RELATIVE_YOFFSET 0x0020
#define DMF_FIXED_XOFFSET    0x0040
#define DMF_FIXED_YOFFSET    0x0080
#define DMF_FIXED_HEIGHT     0x0100
#define DMF_FIXED_WIDTH      0x0200
#define DMF_RELATIVE_HEIGHT  0x0400
#define DMF_RELATIVE_WIDTH   0x0800

#define DMF_HORIZONTAL_FLAGS (DMF_FIXED_WIDTH|DMF_RELATIVE_WIDTH|DMF_FIXED_XOFFSET|DMF_RELATIVE_XOFFSET|DMF_FIXED_Y|DMF_RELATIVE_Y)
#define DMF_VERTICAL_FLAGS (DMF_FIXED_HEIGHT|DMF_RELATIVE_HEIGHT|DMF_FIXED_YOFFSET|DMF_RELATIVE_YOFFSET|DMF_FIXED_X|DMF_RELATIVE_X)

#define DMF_WIDTH_FLAGS (DMF_FIXED_WIDTH|DMF_RELATIVE_WIDTH|DMF_FIXED_XOFFSET|DMF_RELATIVE_XOFFSET)
#define DMF_HEIGHT_FLAGS (DMF_FIXED_HEIGHT|DMF_RELATIVE_HEIGHT|DMF_FIXED_YOFFSET|DMF_RELATIVE_YOFFSET)

/*****************************************************************************
** Universal values for alignment of graphics and text.
*/

#define ALIGN_LEFT       0x0001  /* Align to left */
#define ALIGN_RIGHT      0x0002  /* Align to right */
#define ALIGN_HORIZONTAL 0x0004  /* Align to horizontal center */
#define ALIGN_VERTICAL   0x0008  /* Align to vertical center */
#define ALIGN_TOP        0x0010  /* Align to top */
#define ALIGN_BOTTOM     0x0020  /* Align to bottom */

#define ALIGN_CENTER  (ALIGN_HORIZONTAL|ALIGN_VERTICAL)
#define ALIGN_MIDDLE  (ALIGN_HORIZONTAL|ALIGN_VERTICAL)

/****************************************************************************/

#define FreeFromChain(a,b) if (a->Prev) a->Prev->Next = a->Next; \
                           if (a->Next) a->Next->Prev = a->Prev; \
                           if (a == b) { \
                              b = a->Next; \
                              if (a->Next) a->Next->Prev = 0; \
                           } \
                           a->Prev = 0; \
                           a->Next = 0;

#define FreeSimpleChain(a) if (a->Prev) a->Prev->Next = a->Next; \
                           if (a->Next) a->Next->Prev = a->Prev; \
                           a->Prev = 0; \
                           a->Next = 0;

/*****************************************************************************
** UTF-8 Macros
*/

#define nextutf8(str) if (*(str)) for (++(str); (*(str) & 0xc0) IS 0x80; (str)++);

/*****************************************************************************
** Colour structures.
*/

struct RGB  { LONG  Red, Green, Blue, Alpha; };
struct FRGB { FLOAT Red, Green, Blue, Alpha; };
struct BRGB { UBYTE Red, Green, Blue, Alpha; };

struct HSV {
   DOUBLE Hue;        /* Between 0 and 359 */
   DOUBLE Saturation; /* Between 0 and 100 */
   DOUBLE Value;      /* Between 0 and 100 */
};

/****************************************************************************/

struct FunctionField {
   STRING Name; /* Name of the field */
   LONG Type;   /* Type of the field */
};

/*****************************************************************************
** Action Codes
*/

#define AC_ActionNotify       1
#define AC_Activate           2
#define AC_AccessObject       3
#define AC_Clear              4
#define AC_FreeWarning        5
#define AC_ClosingTag         6
#define AC_CopyData           7
#define AC_DataChannel        8
#define AC_Deactivate         9
#define AC_Draw               10
#define AC_Flush              11
#define AC_Focus              12
#define AC_Free               13
#define AC_ReleaseObject      14
#define AC_GetUnlistedField   15
#define AC_HealthCheck        16
#define AC_Hide               17
#define AC_Init               18
#define AC_Lock               19
#define AC_LostFocus          20
#define AC_Move               21
#define AC_MoveToBack         22
#define AC_MoveToFront        23
#define AC_NewChild           24
#define AC_NewOwner           25
#define AC_NewObject          26
#define AC_OpeningTag         27
#define AC_Query              28
#define AC_Read               29
#define AC_Rename             30
#define AC_Reset              31
#define AC_Resize             32
#define AC_SaveImage          33
#define AC_SaveToObject       34
#define AC_GetField           35
#define AC_Scroll             36
#define AC_Seek               37
#define AC_SetUnlistedField   38
#define AC_Show               39
#define AC_Timer              40
#define AC_Unlock             41
#define AC_UserClick          42
#define AC_UserClickRelease   43
#define AC_UserMovement       44
#define AC_Write              45
#define AC_EventMessage       46
#define AC_FieldNotify        47
#define AC_SetField           48
#define AC_Clipboard          49
#define AC_Refresh            50
#define AC_Disable            51
#define AC_Enable             52
#define AC_Redimension        53
#define AC_UserClickRepeat    54
#define AC_MoveToPoint        55
#define AC_ScrollToPoint      56
#define AC_Custom             57
#define AC_Sort               58
#define AC_END                59

/* NB: Before adding new actions to this list, always consider if it is specific
** enough as to be implemented as a custom action type (AC_Custom).
*/

/****************************************************************************/

#ifndef MODULES_KERNEL_H
#include <pandora/modules/kernel.h>
#endif

#ifndef SYSTEM_ACTIONS_H
#include <pandora/system/actions.h>
#endif

#ifndef SYSTEM_CLASS_H
#include <pandora/system/class.h>
#endif

#endif /* MAIN_H */
